==> lhcsmapi-1.1.6-py3-none-any.whl: lhcsmapi/metadata/Metadata.py <==
import re
import json
import os
from pathlib import Path

class Metadata(object):
    BMODE =  "HX:BMODE"

    __paths = {'RB': 'RbMetadata.json',
               'RQD': 'RqdMetadata.json',
               'RQF': 'RqfMetadata.json',
               '600A': '600AMetadata.json',
               'RQX': 'RqxMetadata.json',
               'IPD2': 'Ipd2Metadata.json',
               'IPQD2': 'Ipqd2Metadata.json',
               'IPQD4': 'Ipqd4Metadata.json',
               'IPQD8': 'Ipqd8Metadata.json',
               }

    @staticmethod
    def createMetadata(metadataKey):

        selectedPath = os.path.dirname(__file__)
        try:
            selectedPath = os.path.join(selectedPath, Metadata.__paths[metadataKey.upper()])
        except KeyError:
            raise KeyError("{} is not mapped to a json file".format(metadataKey))

        return Metadata.__parseJsonFile(selectedPath)

    @staticmethod
    def parseMetadata(metadata, *keys):

        if len(keys) < 1:
            raise ValueError("keys can not be empty!")

        if keys[0] == '*':
            return metadata

        for key in keys:
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist".format(key))

        if len(keys) == 1:
            key = keys[0]
            return {key: metadata[key]}

        return {key: metadata[key] for key in keys}

    @staticmethod
    def getSectorNamesFromComponents(identifier, system, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)['REL_IP_POS_TO_SECTOR']
            if system is "MAGNET":
                regExp = Metadata.createMetadata(identifier)["MAGNET"]['REL_IP_POS_REG_EXP']
            elif system is "BUSBAR":
                regExp = Metadata.createMetadata(identifier)["QPS"]["BUSBAR"]['REL_IP_POS_REG_EXP']
            elif system is "FGC":
                regExp = Metadata.createMetadata(identifier)["FGC"]['REL_IP_POS_REG_EXP']
                if len(keys) == 1:
                    return re.findall(regExp, keys[0])[0]
                else:
                    return [re.findall(regExp, key)[0] for key in keys]

            return Metadata.__getArcNamesFromMagnetNames(metadata, regExp, *keys)
        except KeyError as e:
            raise KeyError(str(e) + "in object parsed from {}".format(Metadata.__paths[identifier]))


    @staticmethod
    def getMetadata(identifier, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)
            return Metadata.parseMetadata(metadata, *keys)
        except KeyError as e:
            raise KeyError(str(e) + ' in object parsed from {}'.format(Metadata.__paths[identifier]))

    @staticmethod
    def __getSectorNameFromMagnetName(metadata, regExp, key):
        foundRegExp = re.findall(regExp, key)
        if len(foundRegExp) != 1:
            raise ValueError("Key {} does not contain proper MB magnet name".format(key))
        else:
            key = foundRegExp[0]
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist ".format(key))
            else:
                return metadata[key]

    @staticmethod
    def __getArcNamesFromMagnetNames(metadata, regExp, *keys):

        if len(keys) == 1:
            return Metadata.__getSectorNameFromMagnetName(metadata, regExp, keys[0])
        else:
            return [Metadata.__getSectorNameFromMagnetName(metadata, regExp, key) for key in keys]

    @staticmethod
    def __parseJsonFile(fileName):

        try:
            with open(fileName) as json_file:
                return json.load(json_file)
        except:
            print("could not load {}".format(fileName))

==> lhcsmapi-1.1.21-py3-none-any.whl: lhcsmapi/metadata/Metadata.py <==
import re
import json
import os
import functools
import numpy as np
import pandas as pd

class Metadata(object):
    BMODE =  "HX:BMODE"

    __paths = {'RB': 'RbMetadata.json',
               'RQD': 'RqdMetadata.json',
               'RQF': 'RqfMetadata.json',
               '600A': '600AMetadata.json',
               'RQX': 'RqxMetadata.json',
               'IPD2': 'Ipd2Metadata.json',
               'IPQD2': 'Ipqd2Metadata.json',
               'IPQD4': 'Ipqd4Metadata.json',
               'IPQD8': 'Ipqd8Metadata.json',
               }

    @staticmethod
    def createMetadata(metadataKey):

        selectedPath = os.path.dirname(__file__)
        try:
            selectedPath = os.path.join(selectedPath, Metadata.__paths[metadataKey.upper()])
        except KeyError:
            raise KeyError("{} is not mapped to a json file".format(metadataKey))

        return Metadata.__parseJsonFile(selectedPath)

    @staticmethod
    def parseMetadata(metadata, *keys):

        if len(keys) < 1:
            raise ValueError("keys can not be empty!")

        if keys[0] == '*':
            return metadata

        for key in keys:
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist".format(key))

        if len(keys) == 1:
            key = keys[0]
            return {key: metadata[key]}

        return {key: metadata[key] for key in keys}

    @staticmethod
    def getBeamModeDetails():
        # https://lhc-commissioning.web.cern.ch/lhc-commissioning/systems/data-exchange/published-topics/beam-modes.htm
        selectedPath = os.path.dirname(__file__)
        fullPath = os.path.join(selectedPath, 'beam_mode/beam_mode.csv')
        return pd.read_csv(fullPath, sep=';')

    @staticmethod
    def getCrateToMagnetAndVoltageFeelerMapping() -> pd.DataFrame:
        # https://wikis.cern.ch/display/MPEEP/DQLPUS+crates+and+magnets
        selectedPath = os.path.dirname(__file__)
        fullPath = os.path.join(selectedPath, 'qps_crate/CrateToMagnetAndVoltageFeelerMap.csv')
        return pd.read_csv(fullPath)
    
    @staticmethod
    def getCrateNameFromMagnetName(magnet: str) -> str:
        crateToMagnetandVf = Metadata.getCrateToMagnetAndVoltageFeelerMapping()
        crateToMagnetandVf = crateToMagnetandVf.dropna()
        mask = functools.reduce(np.logical_or, [crateToMagnetandVf['Magnet{}'.format(i)].str.contains(magnet) for i in range(1, 4)])
        return crateToMagnetandVf.loc[mask]["Crate"].values[0]

    @staticmethod
    def getMagnetToCrateAndVoltageFeelerMapping() -> pd.DataFrame:
        # https://wikis.cern.ch/display/MPEEP/DQLPUS+crates+and+magnets
        selectedPath = os.path.dirname(__file__)
        fullPath = os.path.join(selectedPath, 'qps_crate/MagnetToCrateAndVoltageFeelerMap.csv')
        return pd.read_csv(fullPath)

    @staticmethod
    def getMagnetNamesFromCrateName(crateName: str) -> str:
        magnetToCrateAndVf = Metadata.getMagnetToCrateAndVoltageFeelerMapping()
        magnetToCrateAndVf = magnetToCrateAndVf.dropna()
        magnets = magnetToCrateAndVf[magnetToCrateAndVf["Crate"] == crateName].filter(regex=("Magnet\d{1}")).values[0]
        return [x for x in magnets if "MB." in x]

    @staticmethod
    def getBusbarToMagnetMapping(circuitType):
        if circuitType is "RB":
            fileName = "busbar/nQPS_RB_busBarProtectionDetails.csv"
        elif any(rqType  in circuitType for rqType in ["RQD", "RQF"]):
            fileName = "busbar/nQPS_RQ_busBarProtectionDetails.csv"
        else:
            raise KeyError("Circuit type {} is not supported".format(circuitType))
        selectedPath = os.path.dirname(__file__)
        fullPath = os.path.join(selectedPath, fileName)
        return pd.read_csv(fullPath)

    @staticmethod
    def getSectorNamesFromComponents(identifier, system, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)['REL_IP_POS_TO_CIRCUIT_NAME']
            if system is "MAGNET":
                regExp = Metadata.createMetadata(identifier)["MAGNET"]['REL_IP_POS_REG_EXP']
            elif system is "BUSBAR":
                regExp = Metadata.createMetadata(identifier)["QPS"]["BUSBAR"]['REL_IP_POS_REG_EXP']
            elif system is "FGC":
                regExp = Metadata.createMetadata(identifier)["FGC"]['REL_IP_POS_REG_EXP']
                if len(keys) == 1:
                    return re.findall(regExp, keys[0])[0]
                else:
                    return [re.findall(regExp, key)[0] for key in keys]

            return Metadata.__getArcNamesFromMagnetNames(metadata, regExp, *keys)
        except KeyError as e:
            raise KeyError(str(e) + "in object parsed from {}".format(Metadata.__paths[identifier]))


    @staticmethod
    def getMetadata(identifier, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)
            return Metadata.parseMetadata(metadata, *keys)
        except KeyError as e:
            raise KeyError(str(e) + ' in object parsed from {}'.format(Metadata.__paths[identifier]))

    @staticmethod
    def __getSectorNameFromMagnetName(metadata, regExp, key):
        foundRegExp = re.findall(regExp, key)
        if len(foundRegExp) != 1:
            raise ValueError("Key {} does not contain proper MB magnet name".format(key))
        else:
            key = foundRegExp[0]
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist ".format(key))
            else:
                return metadata[key]

    @staticmethod
    def __getArcNamesFromMagnetNames(metadata, regExp, *keys):

        if len(keys) == 1:
            return Metadata.__getSectorNameFromMagnetName(metadata, regExp, keys[0])
        else:
            return [Metadata.__getSectorNameFromMagnetName(metadata, regExp, key) for key in keys]

    @staticmethod
    def __parseJsonFile(fileName):

        try:
            with open(fileName) as json_file:
                return json.load(json_file)
        except:
            print("Could not load {}".format(fileName))
